#include "m_pd.h"
#include <stdio.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>
#include <errno.h>
#include <linux/i2c-dev.h>
#include <math.h>
#include <stdlib.h>

int i2c_init(char filename[40], int addr);
int decimal_binary(int n);
int binary_decimal(int n);

__s32 byteval1;
__s32 byteval2;
__s32 byteval3;
__s32 byteval4;
t_float pot1 = 0;
t_float pot2 = 0;
t_float pot3 = 0;
t_float pot4 = 0;

static t_class *i2c_PCF8591_class;

typedef struct _i2c_PCF8591 {
  t_object  x_obj;
  char buf[1024];
  char buf2[1024];
  t_int int_addr;
  t_int slave;
  t_outlet *out1, *out2, *out3, *out4;
} t_i2c_PCF8591;

static void i2c_PCF8591_bang(t_i2c_PCF8591 *x){

  /* Using SMBus commands */
  byteval1 = i2c_smbus_read_byte_data(x->slave, 0x40);
  byteval1 = i2c_smbus_read_byte_data(x->slave, 0x40);  
  byteval2 = i2c_smbus_read_byte_data(x->slave, 0x41);
  byteval2 = i2c_smbus_read_byte_data(x->slave, 0x41);  
  byteval3 = i2c_smbus_read_byte_data(x->slave, 0x42);
  byteval3 = i2c_smbus_read_byte_data(x->slave, 0x42);  
  byteval4 = i2c_smbus_read_byte_data(x->slave, 0x43);
  byteval4 = i2c_smbus_read_byte_data(x->slave, 0x43);  
  
  pot1=byteval1; /* type cast */ 
  pot2=byteval2;
  pot3=byteval3;
  pot4=byteval4;

  outlet_float(x->out1, pot1);
  outlet_float(x->out2, pot2);
  outlet_float(x->out3, pot3);
  outlet_float(x->out4, pot4);
}

static void *i2c_PCF8591_new(t_symbol *nom_arg, t_symbol *addr1){
  t_i2c_PCF8591 *x = (t_i2c_PCF8591 *)pd_new(i2c_PCF8591_class);
  
  x->int_addr = (int)strtol(addr1->s_name, NULL, 0);
  sprintf(x->buf, "%s",nom_arg->s_name);
  sprintf(x->buf2, "%s",addr1->s_name);

  x->slave=i2c_init(x->buf,x->int_addr); //dev,slavei2caddr

  /* test connection */
  byteval1 = i2c_smbus_read_byte_data(x->slave, 0x40);
  byteval2 = i2c_smbus_read_byte_data(x->slave, 0x41);  
  byteval3 = i2c_smbus_read_byte_data(x->slave, 0x42);
  byteval4 = i2c_smbus_read_byte_data(x->slave, 0x43);  

  if (byteval1 < 0) {
  /* ERROR HANDLING: i2c transaction failed */
  post("IN1 transaction failed.");
  } else {
    pot1=byteval1;
  }
  if (byteval2 < 0) {
  /* ERROR HANDLING: i2c transaction failed */
  post("IN2 transaction failed.");
  } else {
    pot2=byteval2;
  }
  if (byteval3 < 0) {
  /* ERROR HANDLING: i2c transaction failed */
  post("IN3 transaction failed.");
  } else {
    pot3=byteval3;
  }
  if (byteval4 < 0) {
  /* ERROR HANDLING: i2c transaction failed */
  post("IN4 transaction failed.");
  } else {
    pot4=byteval4;
  }
  
  x->out1 = outlet_new(&x->x_obj, &s_float);
  x->out2 = outlet_new(&x->x_obj, &s_float);
  x->out3 = outlet_new(&x->x_obj, &s_float);
  x->out4 = outlet_new(&x->x_obj, &s_float);  
  return (void *)x;
}

void i2c_PCF8591_setup(void) {
  i2c_PCF8591_class = class_new(gensym("i2c_PCF8591"),
        (t_newmethod)i2c_PCF8591_new,
        0, sizeof(t_i2c_PCF8591),
        CLASS_DEFAULT,
        A_DEFSYMBOL,A_DEFSYMBOL, 0);

  class_addbang(i2c_PCF8591_class, i2c_PCF8591_bang);
}

void i2c_PCF8591_destroy(t_i2c_PCF8591 *x) {
  close(x->slave);
}


// i2c functions

int i2c_init(char filename[40], int addr){
  int file;
 
  if ((file = open(filename,O_RDWR)) < 0){
    post("Failed to open the bus.\n");
  }
 
  if (ioctl(file,I2C_SLAVE,addr) < 0){
    post("Failed to acquire bus access and/or talk to slave.\n");
  }
  return file;
}

// conversion functions

int decimal_binary(int n)  /* Function to convert decimal to binary.*/
{
  int rem, i=1, binary=0;
  while (n!=0)
  {
    rem=n%2;
    n/=2;
    binary+=rem*i;
    i*=10;
  }
  return binary;
}

int binary_decimal(int n) /* Function to convert binary to decimal.*/

{
  int decimal=0, i=0, rem;
  while (n!=0)
  {
    rem = n%10;
    n/=10;
    decimal += rem*pow(2,i);
    ++i;
  }
  return decimal;
}

